/* Inline frame unwinder for GDB.

   Copyright (C) 2007 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "defs.h"
#include "addrmap.h"
#include "block.h"
#include "frame-unwind.h"
#include "symtab.h"

#include "gdb_assert.h"

static int inline_skip_frames;

/* Only valid if INLINE_SKIP_FRAMES is non-zero.  This is the PC used
   when calculating INLINE_SKIP_FRAMES; used to check whether we have
   moved to a new location by user request.  */
static CORE_ADDR inline_skip_pc;

/* Only valid if INLINE_SKIP_FRAMES is non-zero.  This is the symbol
   of the outermost skipped inline function.  It's used to find the
   call site of the current frame.  */
static struct symbol *inline_skip_symbol;

/* Transparently wrap the former regular unwinder.  Allocated by
   FRAME_OBSTACK_ZALLOC.  */
struct inline_cache
  {
    const struct frame_unwind *unwind_regular;
    void *cache_regular;
  };

/* We just must remap wrapped THIS_CACHE.  */

static void
inline_frame_dealloc (struct frame_info *self, void *this_cache)
{
  struct inline_cache *inline_cache = this_cache;

  /* The same condition as in REINIT_FRAME_CACHE.  */
  if (inline_cache->unwind_regular->dealloc_cache
      && inline_cache->cache_regular)
    inline_cache->unwind_regular->dealloc_cache (self,
						 inline_cache->cache_regular);
}

static CORE_ADDR
inline_frame_prev_pc (struct frame_info *next_frame, void **this_prologue_cache)
{
  struct inline_cache *inline_cache = *this_prologue_cache;
  struct frame_info *this_frame;

  if (inline_cache->unwind_regular->prev_pc)
    return inline_cache->unwind_regular->prev_pc (next_frame,
						  &inline_cache->cache_regular);
  gdb_assert (gdbarch_unwind_pc_p (get_frame_arch (next_frame)));
  this_frame = get_prev_frame (next_frame);
  gdb_assert (this_frame);
  gdb_assert (get_frame_arch (next_frame) == get_frame_arch (this_frame));
  return gdbarch_unwind_pc (get_frame_arch (this_frame), this_frame);
}

static void
inline_frame_this_id (struct frame_info *this_frame,
		      void **this_cache,
		      struct frame_id *this_id)
{
  struct symbol *func;
  struct frame_info *next_frame;
  struct inline_cache *inline_cache = *this_cache;

  this_frame = get_prev_frame (this_frame);
  gdb_assert (get_frame_type (this_frame) == INLINE_FRAME);

  /* In order to have a stable frame ID for a given inline function, we must
     get the stack / special addresses from the underlying real frame's this_id
     method.  We must not call get_prev_frame as each frame expects its inner
     frames to be already fully accessible.  This would open a can of worms
     with unwinding deadlocks.  */
  inline_cache->unwind_regular->this_id (this_frame,
					 &inline_cache->cache_regular, this_id);

  /* We need a valid frame ID, so we need to be based on a valid
     frame.  FSF submission NOTE: this would be a good assertion to
     apply to all frames, all the time.  That would fix the ambiguity
     of null_frame_id (between "no/any frame" and "the outermost
     frame").  This will take work.  */
  gdb_assert (frame_id_p (*this_id));

  /* Future work NOTE: Alexandre Oliva applied a patch to GCC 4.3
     which generates DW_AT_entry_pc for inlined functions when
     possible.  If this attribute is available, we should use it
     in the frame ID (and eventually, to set breakpoints).  */
  func = get_frame_function (this_frame);
  gdb_assert (func != NULL);
  (*this_id).block_addr = BLOCK_START (SYMBOL_BLOCK_VALUE (func));

  /* The regular unwinder knows no INLINE_DEPTH.  */
  gdb_assert ((*this_id).inline_depth == 0);

  /* Inlined frames (inlined functions) can never exist behind a normal frame
     (non-inlined function).  Any possible inlined frames (inlined functions)
     behind such frame will already be at a different code location and not
     found by BLOCK_FOR_PC at this PC moment.  */
  for (next_frame = this_frame;
       next_frame && get_frame_type (next_frame) == INLINE_FRAME;
       next_frame = get_next_frame (next_frame))
    (*this_id).inline_depth++;

  gdb_assert ((*this_id).inline_depth > 0);
}

static void
inline_frame_prev_register (struct frame_info *next_frame,
			    void **this_prologue_cache,
			    int prev_regnum,
			    int *optimized,
			    enum lval_type * lvalp,
			    CORE_ADDR *addrp,
			    int *realnump, gdb_byte *valuep)
{
  /* Copy all the registers unchanged.  */
  *optimized = 0;
  *lvalp = lval_register;
  *addrp = 0;
  *realnump = prev_regnum;
  if (valuep)
    frame_unwind_register (next_frame, (*realnump), valuep);
}

/* Check whether we are at an inlining site that does not already
   have an associated frame.  */

static int
inline_frame_sniffer (const struct frame_unwind *self,
		      struct frame_info *this_frame,
		      void **this_cache)
{
  CORE_ADDR this_pc;
  struct block *frame_block, *cur_block;
  int depth;
  struct frame_info *next_frame;
  struct inline_cache *inline_cache;

  /* INLINE_FRAME unwinder is an exception, it is passed the regular unwinder
     detected for this frame.  */
  gdb_assert (self != inline_frame_unwind);

  this_pc = frame_unwind_address_in_block (this_frame,
					   get_frame_type (this_frame));
  frame_block = block_for_pc (this_pc);
  if (frame_block == NULL)
    return 0;

  /* INLINE_SKIP_SYMBOL does not need to be set for each specific frame.  But
   * it needs to be recalculated after STEP_INTO_INLINE_FRAME according to new
   * INLINE_SKIP_FRAMES - therefore SET_SKIPPED_INLINE_FRAMES is too early.  */
  inline_skip_symbol = NULL;

  /* Calculate DEPTH, the number of inlined functions at this
     location.  */
  depth = 0;
  cur_block = frame_block;
  while (BLOCK_SUPERBLOCK (cur_block))
    {
      if (block_inlined_p (cur_block))
	depth++;
      if (depth == inline_skip_frames && inline_skip_symbol == NULL)
	inline_skip_symbol = BLOCK_FUNCTION (cur_block);

      cur_block = BLOCK_SUPERBLOCK (cur_block);
    }

  /* There are inlined functions here.  Check how many of them already have
     frames.  Any possibly inlined frames (inlined functions) behind
     NORMAL_FRAME (non-inlined function) will already be at a different code
     location and not found by BLOCK_FOR_PC at this PC moment.  */
  if (get_frame_type (this_frame) == SENTINEL_FRAME)
    this_frame = NULL;
  for (next_frame = this_frame;
       next_frame && get_frame_type (next_frame) == INLINE_FRAME;
       next_frame = get_next_frame (next_frame))
    {
      gdb_assert (depth > 0);
      depth--;
    }

  /* Check whether we were requested to skip some frames, so they
     can be stepped into later.  */
  if (inline_skip_frames > 0 && next_frame == NULL)
    {
      if (this_pc != inline_skip_pc)
	inline_skip_frames = 0;
      else
	{
	  gdb_assert (depth >= inline_skip_frames);
	  depth -= inline_skip_frames;
	}
    }

  /* If all the inlined functions already have frames, then pass to the
     normal unwinder for this PC.  */
  if (depth == 0)
    return 0;

  /* Transparently wrap the former regular unwinder.  */
  inline_cache = frame_obstack_zalloc (sizeof (*inline_cache));
  inline_cache->unwind_regular = self;
  inline_cache->cache_regular = *this_cache;
  *this_cache = inline_cache;

  /* If the next frame is an inlined function, but not the outermost, then
     we are the next outer.  If it is not an inlined function, then we
     are the innermost inlined function of a different real frame.  */
  return 1;
}

static const struct frame_unwind inline_frame_unwinder = {
  INLINE_FRAME,
  inline_frame_this_id,
  inline_frame_prev_register,
  NULL,
  inline_frame_sniffer,
  inline_frame_prev_pc,
  inline_frame_dealloc
};

const struct frame_unwind *const inline_frame_unwind = {
    &inline_frame_unwinder
};

/* Return non-zero if BLOCK, an inlined function block containing PC,
   has a group of contiguous instructions starting at PC (but not
   before it).  */

static int
block_starting_point_at (CORE_ADDR pc, struct block *block)
{
  struct blockvector *bv;
  struct block *new_block;

  bv = blockvector_for_pc (pc, NULL);
  if (BLOCKVECTOR_MAP (bv) == NULL)
    return 0;

  new_block = addrmap_find (BLOCKVECTOR_MAP (bv), pc - 1);
  if (new_block == NULL)
    return 1;

  if (new_block == block || contained_in (new_block, block))
    return 0;

  /* The immediately preceeding address belongs to a different block,
     which is not a child of this one.  Treat this as an entrance into
     BLOCK.  */
  return 1;
}

void
set_skipped_inline_frames (int skip_ok)
{
  CORE_ADDR this_pc;
  struct block *frame_block, *cur_block;
  int skip_count = 0;

  if (!skip_ok)
    {
      if (inline_skip_frames != 0)
	{
	  inline_skip_frames = 0;
	  reinit_frame_cache ();
	}
      return;
    }

  /* This function is called right after reinitializing the frame
     cache.  We try not to do more unwinding than absolutely
     necessary, for performance.  */
  this_pc = get_frame_pc (get_current_user_frame ());
  frame_block = block_for_pc (this_pc);

  if (frame_block != NULL)
    {
      cur_block = frame_block;
      while (BLOCK_SUPERBLOCK (cur_block))
	{
	  if (block_inlined_p (cur_block))
	    {
	      /* See comments in inline_frame_this_id about this use
		 of BLOCK_START.  */
	      if (BLOCK_START (cur_block) == this_pc
		  || block_starting_point_at (this_pc, cur_block))
		skip_count++;
	      else
		{
		  /* Here we will stop at any possible inlining after we
		     already crossed any first non-inlined function.  We could
		     possibly detected such functions generating NORMAL_FRAME
		     by `!block_inlined_p && BLOCK_FUNCTION' (as
		     `!block_inlined_p && !BLOCK_FUNCTION' are just anonymous
		     local { ... } blocks).  */
		  break;
		}
	    }
	  cur_block = BLOCK_SUPERBLOCK (cur_block);
	}
    }

  inline_skip_pc = this_pc;

  if (inline_skip_frames != skip_count)
    {
      inline_skip_frames = skip_count;
      reinit_frame_cache ();
    }
}

void
step_into_inline_frame (void)
{
  gdb_assert (inline_skip_frames > 0);
  inline_skip_frames--;
  reinit_frame_cache ();
}

int
inline_skipped_frames (void)
{
  return inline_skip_frames;
}

struct symbol *
inline_skipped_symbol (void)
{
  return inline_skip_symbol;
}
